import json
import six
from pyamf.remoting import Envelope, Request, decode
from pyamf.flex.messaging import RemotingMessage
from io import BytesIO
from six.moves.urllib.parse import urlencode


class Amf:
    @staticmethod
    def get_representation(request_object, max_calls=500):

        max_calls = max_calls - 1

        if max_calls < 0:
            raise Exception("Amf.get_representation maximum number of calls reached")

        if isinstance(request_object, Envelope):
            # Remove order of Request
            bodies = []
            for i in request_object.bodies:
                bodies.append(Amf.get_representation(i[1], max_calls))
            bodies = sorted(bodies)

            return "<Envelope>{bodies}</Envelope>".format(
                bodies="[" + ",".join(bodies) + "]"
            )

        elif isinstance(request_object, Request):
            # Remove cyclic reference
            target = request_object.target
            body = Amf.get_representation(request_object.body, max_calls)
            return "<Request target={target}>{body}</Request>".format(**locals())

        elif isinstance(request_object, RemotingMessage):
            # Remove random properties
            operation = request_object.operation
            body = Amf.get_representation(request_object.body, max_calls)
            return "<RemotingMessage operation={operation}>{body}</RemotingMessage>".format(
                **locals()
            )

        elif isinstance(request_object, dict):
            return json.dumps(request_object, sort_keys=True)

        elif isinstance(request_object, list):
            bodies = []
            for i in request_object:
                bodies.append(Amf.get_representation(i, max_calls))
            return "[" + ",".join(bodies) + "]"

        elif isinstance(request_object, six.string_types):
            return request_object

        elif request_object is None:
            return ""

        elif isinstance(request_object, object) and hasattr(request_object, "__dict__"):
            classname = request_object.__class__.__name__
            properties = request_object.__dict__
            bodies = dict()
            for prop in properties:
                bodies[prop] = Amf.get_representation(
                    getattr(request_object, prop), max_calls
                )
            bodies = Amf.get_representation(bodies, max_calls)

            return "<{classname}>{bodies}</{classname}>".format(**locals())

        else:
            return repr(request_object)


def amf_parse(string):
    try:
        res = decode(BytesIO(string))
        return urlencode({"request": Amf.get_representation(res)})

    except Exception as e:
        import traceback

        traceback.print_exc()
        print(e)
        return None



from io import BytesIO

from pyamf import AMF3
from pyamf.remoting import Request, Envelope, encode

from cdxj_indexer.postquery import append_method_query
from cdxj_indexer.amf import amf_parse


# ============================================================================
class MethodQueryCanonicalizer:
    def __init__(self, method, content_type, req_len, req_stream):
        self.method = method
        self.content_type = content_type
        self.req_len = req_len
        self.req_stream = req_stream

    def append_query(self, url):
        self.req_stream.seek(0)
        query_only, full_string = append_method_query(
            self.method, self.content_type, self.req_len, self.req_stream, url
        )
        return url + full_string


# ============================================================================
class TestPostQueryExtract(object):
    @classmethod
    def setup_class(cls):
        cls.post_data = b"foo=bar&dir=%2Fbaz"
        cls.binary_post_data = (
            b"\x816l`L\xa04P\x0e\xe0r\x02\xb5\x89\x19\x00fP\xdb\x0e\xb0\x02,"
        )

    def test_post_extract_1(self):
        mq = MethodQueryCanonicalizer(
            "POST",
            "application/x-www-form-urlencoded",
            len(self.post_data),
            BytesIO(self.post_data),
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&foo=bar&dir=/baz"
        )

        assert (
            mq.append_query("http://example.com/?123=ABC")
            == "http://example.com/?123=ABC&__wb_method=POST&foo=bar&dir=/baz"
        )

    def test_post_extract_json(self):
        post_data = b'{"a": "b", "c": {"a": 2}, "d": "e"}'
        mq = MethodQueryCanonicalizer(
            "POST", "application/json", len(post_data), BytesIO(post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&a=b&a.2_=2&d=e"
        )

    def test_post_extract_json_top_list(self):
        post_data = (
            b'[{"a": "b", "c": {"a": 2}}, {"d": "e"}, "ignored", false, null, 0]'
        )
        mq = MethodQueryCanonicalizer(
            "POST", "application/json", len(post_data), BytesIO(post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&a=b&a.2_=2&d=e"
        )

    def test_post_extract_json_lines(self):
        post_data = b'{"a": "b"}\n{"c": {"a": 2}, "d": "e"}'
        mq = MethodQueryCanonicalizer(
            "POST", "application/json", len(post_data), BytesIO(post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&a=b&a.2_=2&d=e"
        )

    def test_put_extract_method(self):
        mq = MethodQueryCanonicalizer(
            "PUT",
            "application/x-www-form-urlencoded",
            len(self.post_data),
            BytesIO(self.post_data),
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=PUT&foo=bar&dir=/baz"
        )

    def test_post_extract_non_form_data_1(self):
        mq = MethodQueryCanonicalizer(
            "POST",
            "application/octet-stream",
            len(self.post_data),
            BytesIO(self.post_data),
        )

        # base64 encoded data
        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&__wb_post_data=Zm9vPWJhciZkaXI9JTJGYmF6"
        )

    def test_post_extract_non_form_data_2(self):
        mq = MethodQueryCanonicalizer(
            "POST", "text/plain", len(self.post_data), BytesIO(self.post_data)
        )

        # base64 encoded data
        assert (
            mq.append_query("http://example.com/pathbar?id=123")
            == "http://example.com/pathbar?id=123&__wb_method=POST&__wb_post_data=Zm9vPWJhciZkaXI9JTJGYmF6"
        )

    def test_post_extract_length_invalid_ignore(self):
        mq = MethodQueryCanonicalizer(
            "POST", "application/x-www-form-urlencoded", 0, BytesIO(self.post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST"
        )

        mq = MethodQueryCanonicalizer(
            "POST", "application/x-www-form-urlencoded", "abc", BytesIO(self.post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST"
        )

    def test_post_extract_length_too_short(self):
        mq = MethodQueryCanonicalizer(
            "POST",
            "application/x-www-form-urlencoded",
            len(self.post_data) - 4,
            BytesIO(self.post_data),
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&foo=bar&dir=%2"
        )

    def test_post_extract_length_too_long(self):
        mq = MethodQueryCanonicalizer(
            "POST",
            "application/x-www-form-urlencoded",
            len(self.post_data) + 4,
            BytesIO(self.post_data),
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&foo=bar&dir=/baz"
        )

    def test_post_extract_malformed_form_data(self):
        mq = MethodQueryCanonicalizer(
            "POST",
            "application/x-www-form-urlencoded",
            len(self.binary_post_data),
            BytesIO(self.binary_post_data),
        )

        # base64 encoded data
        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&__wb_post_data=gTZsYEygNFAO4HICtYkZAGZQ2w6wAiw="
        )

    def test_post_extract_no_boundary_in_multipart_form_mimetype(self):
        mq = MethodQueryCanonicalizer(
            "POST", "multipart/form-data", len(self.post_data), BytesIO(self.post_data)
        )

        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=POST&__wb_post_data=Zm9vPWJhciZkaXI9JTJGYmF6"
        )

    def test_options(self):
        mq = MethodQueryCanonicalizer("OPTIONS", "", 0, BytesIO())
        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=OPTIONS"
        )

    def test_head(self):
        mq = MethodQueryCanonicalizer("HEAD", "", 0, BytesIO())
        assert (
            mq.append_query("http://example.com/")
            == "http://example.com/?__wb_method=HEAD"
        )

    def test_amf_parse(self):
        mq = MethodQueryCanonicalizer("POST", "application/x-amf", 0, BytesIO())

        req = Request(target="t", body="")
        ev_1 = Envelope(AMF3)
        ev_1["/0"] = req

        req = Request(target="t", body="alt_content")
        ev_2 = Envelope(AMF3)
        ev_2["/0"] = req

        assert amf_parse(encode(ev_1).getvalue()) != amf_parse(encode(ev_2).getvalue())



import os
import sys
import tempfile
from io import BytesIO

try:
    from StringIO import StringIO
except ImportError:  # pragma: no cover
    from io import StringIO

from cdxj_indexer.main import write_cdx_index, main, CDXJIndexer

import pkg_resources

TEST_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")


# ============================================================================
class TestIndexing(object):
    def index_file(self, filename, **opts):
        output = StringIO()
        write_cdx_index(output, os.path.join(TEST_DIR, filename), opts)
        return output.getvalue()

    def index_all(self, filenames, **opts):
        output = StringIO()
        # paths = [os.path.join(TEST_DIR, filename) for filename in os.listdir(TEST_DIR)]
        paths = [os.path.join(TEST_DIR, filename) for filename in filenames]
        write_cdx_index(output, paths, opts)
        return output.getvalue()

    def index_file_cli(self, filename, capsys, extra_params=None):
        params = [os.path.join(TEST_DIR, filename)]
        if extra_params:
            params += extra_params

        res = main(params)

        return capsys.readouterr().out

    def test_warc_cdxj(self):
        res = self.index_file("example.warc.gz")
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz"}
"""
        assert res == exp

    def test_warc_cdxj_cli_main(self, capsys):
        res = self.index_file_cli("example.warc.gz", capsys)
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz"}
"""
        assert res == exp

    def test_warc_cdxj_with_record_digests(self):
        res = self.index_file("example.warc.gz", digest_records=True, post_append=True)
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz", "recordDigest": "sha256:1ebd93871e6de75fb72d5398452e4152cf3d655ae31368e7336c1b57870d244c"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz", "recordDigest": "sha256:f67dfffc2d78d025030574b08ad57211d2d2211ae49d566adf7868072a823f74"}
"""
        assert res == exp

    def test_warc_cdxj_cli_main_with_record_digests(self, capsys):
        res = self.index_file_cli("example.warc.gz", capsys, ["-d", "-p"])
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz", "recordDigest": "sha256:1ebd93871e6de75fb72d5398452e4152cf3d655ae31368e7336c1b57870d244c"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz", "recordDigest": "sha256:f67dfffc2d78d025030574b08ad57211d2d2211ae49d566adf7868072a823f74"}
"""
        assert res == exp

    def test_warc_cdxj_sorted(self):
        res = self.index_file("cc.warc.gz", sort=True)
        exp = """\
org,commoncrawl)/ 20170722005011 {"url": "https://commoncrawl.org/", "mime": "text/html", "status": "200", "digest": "sha1:RXZILWL37W7MAZTH76FEVIHSF2DZ5HTM", "length": "5357", "offset": "377", "filename": "cc.warc.gz"}
"""
        assert res == exp

    def test_warc_cdxj_dir_root(self):
        res = self.index_file("example.warc.gz", dir_root="./")
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "test/data/example.warc.gz"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "test/data/example.warc.gz"}
"""
        assert res == exp

    def test_warc_cdx_11(self):
        res = self.index_file("example.warc.gz", cdx11=True)
        exp = """\
 CDX N b a m s k r M S V g
com,example)/ 20170306040206 http://example.com/ text/html 200 G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK - - 1242 784 example.warc.gz
com,example)/ 20170306040348 http://example.com/ warc/revisit 200 G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK - - 585 2635 example.warc.gz
"""
        assert res == exp

    def test_warc_cdx_9(self):
        res = self.index_file("example.warc.gz", cdx09=True)
        exp = """\
 CDX N b a m s k r V g
com,example)/ 20170306040206 http://example.com/ text/html 200 G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK - 784 example.warc.gz
com,example)/ 20170306040348 http://example.com/ warc/revisit 200 G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK - 2635 example.warc.gz
"""
        assert res == exp

    def test_warc_cdx_11_avoid_dupe_line(self):
        res = self.index_file("", cdx11=True, sort=True)
        lines = res.split("\n")
        assert lines[0] == " CDX N b a m s k r M S V g"
        assert lines[1] != " CDX N b a m s k r M S V g"

    def test_index_multiple_files(self):
        res = self.index_all(["example.warc.gz", "post-test.warc.gz"])
        assert len(res.strip().split("\n")) == 5

    def test_warc_request_only(self):
        res = self.index_file("example.warc.gz", records="request", fields="method")
        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "digest": "sha1:3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ", "length": "609", "offset": "2026", "filename": "example.warc.gz", "method": "GET"}
com,example)/ 20170306040348 {"url": "http://example.com/", "digest": "sha1:3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ", "length": "609", "offset": "3220", "filename": "example.warc.gz", "method": "GET"}
"""
        assert res == exp

    def test_warc_all_cdxj(self):
        res = self.index_file("example.warc.gz", records="all")
        exp = """\
- 20170306040353 {"mime": "application/warc-fields", "length": "353", "offset": "0", "filename": "example.warc.gz"}
- 20170306040353 {"mime": "application/warc-fields", "length": "431", "offset": "353", "filename": "example.warc.gz"}
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz"}
com,example)/ 20170306040206 {"url": "http://example.com/", "digest": "sha1:3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ", "length": "609", "offset": "2026", "filename": "example.warc.gz"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz"}
com,example)/ 20170306040348 {"url": "http://example.com/", "digest": "sha1:3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ", "length": "609", "offset": "3220", "filename": "example.warc.gz"}
"""
        assert res == exp

        res = self.index_file("example.warc.gz", records="all", post_append=True)
        assert res == exp

    def test_arc_cdxj(self):
        res = self.index_file("example.arc")
        exp = """\
com,example)/ 20140216050221 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:B2LTWWPUOYAH7UIPQ7ZUPQ4VMBSVC36A", "length": "1656", "offset": "151", "filename": "example.arc"}
"""
        assert res == exp

    def test_arc_bad_edgecase(self):
        res = self.index_file("bad.arc", cdx11=True, post_append=True)
        exp = """\
 CDX N b a m s k r M S V g
com,example)/ 20140401000000 http://example.com/ - - 3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ - - 67 134 bad.arc
com,example)/ 20140102000000 http://example.com/ - - 3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ - - 59 202 bad.arc
com,example)/ 20140401000000 http://example.com/ - - 3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ - - 68 262 bad.arc
"""
        assert res == exp

    def test_warc_post_query_append(self):
        res = self.index_file("post-test.warc.gz", post_append=True)
        exp = """\
org,httpbin)/post?__wb_method=post&foo=bar&test=abc 20140610000859 {"url": "http://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:M532K5WS4GY2H4OVZO6HRPOP47A7KDWU", "length": "720", "offset": "0", "filename": "post-test.warc.gz", "requestBody": "foo=bar&test=abc", "method": "POST"}
org,httpbin)/post?__wb_method=post&a=1&b=[]&c=3 20140610001151 {"url": "http://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:M7YCTM7HS3YKYQTAWQVMQSQZBNEOXGU2", "length": "723", "offset": "1196", "filename": "post-test.warc.gz", "requestBody": "A=1&B=[]&C=3", "method": "POST"}
org,httpbin)/post?__wb_method=post&data=^&foo=bar 20140610001255 {"url": "http://httpbin.org/post?foo=bar", "mime": "application/json", "status": "200", "digest": "sha1:B6E5P6JUZI6UPDTNO4L2BCHMGLTNCUAJ", "length": "723", "offset": "2395", "filename": "post-test.warc.gz", "requestBody": "data=^", "method": "POST"}
"""
        assert res == exp

        res = self.index_file("post-test.warc.gz")
        exp = """\
org,httpbin)/post 20140610000859 {"url": "http://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:M532K5WS4GY2H4OVZO6HRPOP47A7KDWU", "length": "720", "offset": "0", "filename": "post-test.warc.gz"}
org,httpbin)/post 20140610001151 {"url": "http://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:M7YCTM7HS3YKYQTAWQVMQSQZBNEOXGU2", "length": "723", "offset": "1196", "filename": "post-test.warc.gz"}
org,httpbin)/post?foo=bar 20140610001255 {"url": "http://httpbin.org/post?foo=bar", "mime": "application/json", "status": "200", "digest": "sha1:B6E5P6JUZI6UPDTNO4L2BCHMGLTNCUAJ", "length": "723", "offset": "2395", "filename": "post-test.warc.gz"}
"""
        assert res == exp

    def test_warc_post_query_append_multi_and_json(self):
        res = self.index_file("post-test-more.warc", post_append=True)
        exp = """\
org,httpbin)/post?__wb_method=post&another=more^data&test=some+data 20200809195334 {"url": "https://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:7AWVEIPQMCA4KTCNDXWSZ465FITB7LSK", "length": "688", "offset": "0", "filename": "post-test-more.warc", "requestBody": "test=some+data&another=more%5Edata", "method": "POST"}
org,httpbin)/post?__wb_method=post&a=json-data 20200809195334 {"url": "https://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:BYOQWRSQFW3A5SNUBDSASHFLXGL4FNGB", "length": "655", "offset": "1227", "filename": "post-test-more.warc", "requestBody": "a=json-data", "method": "POST"}
org,httpbin)/post?__wb_method=post&__wb_post_data=c29tzwnodw5rlwvuy29kzwrkyxrh 20200810055049 {"url": "https://httpbin.org/post", "mime": "application/json", "status": "200", "digest": "sha1:34LEADQD3MOBQ42FCO2WA5TUSEL5QOKP", "length": "628", "offset": "2338", "filename": "post-test-more.warc", "requestBody": "__wb_post_data=c29tZWNodW5rLWVuY29kZWRkYXRh", "method": "POST"}
"""
        assert res == exp

    def test_warc_cdxj_compressed_1(self):
        # specify file directly
        with tempfile.TemporaryFile() as temp_fh:
            res = self.index_file(
                "",
                sort=True,
                post_append=True,
                compress=temp_fh,
                data_out_name="comp.cdxj.gz",
                lines=11,
            )

        exp = """\
!meta 0 {"format": "cdxj-gzip-1.0", "filename": "%s"}
com,example)/ 20140102000000 {"offset": 0, "length": 790}
org,httpbin)/post?__wb_method=post&data=^&foo=bar 20140610001255 {"offset": 790, "length": 325}
"""
        assert res == exp % "comp.cdxj.gz"

        # specify named temp file, extension auto-added
        with tempfile.NamedTemporaryFile() as temp_fh:
            res = self.index_file(
                "", sort=True, post_append=True, compress=temp_fh.name, lines=11
            )
            name = temp_fh.name

        assert res == exp % (name + ".cdxj.gz")

        # specify named temp file, with extension suffix
        with tempfile.NamedTemporaryFile(suffix=".cdxj.gz") as temp2_fh:
            res = self.index_file(
                "", sort=True, post_append=True, compress=temp2_fh.name, lines=11
            )
            name = temp2_fh.name

        assert res == exp % name

    def test_warc_cdxj_compressed_2_with_digest(self):
        # specify file directly
        with tempfile.TemporaryFile() as temp_fh:
            res = self.index_file(
                "",
                sort=True,
                post_append=True,
                compress=temp_fh,
                data_out_name="comp_2.cdxj.gz",
                lines=11,
                digest_records=True,
            )

        lines = res.strip().split("\n")

        assert len(lines) == 3
        assert (
            lines[0]
            == '!meta 0 {"format": "cdxj-gzip-1.0", "filename": "comp_2.cdxj.gz"}'
        )
        assert lines[1].startswith(
            'com,example)/ 20140102000000 {"offset": 0, "length": 1292, "digest": "sha256:'
        )
        assert lines[2].startswith(
            'org,httpbin)/post?__wb_method=post&data=^&foo=bar 20140610001255 {"offset": 1292, "length": 428, "digest": "sha256:'
        )

        # specify named temp file, extension auto-added
        with tempfile.NamedTemporaryFile() as temp_fh:
            res2 = self.index_file(
                "",
                sort=True,
                post_append=True,
                compress=temp_fh.name,
                lines=11,
                digest_records=True,
            )
            name = temp_fh.name

        assert res2 == res.replace("comp_2", name)

        # specify named temp file, with extension suffix
        with tempfile.NamedTemporaryFile(suffix=".cdxj.gz") as temp2_fh:
            res3 = self.index_file(
                "",
                sort=True,
                post_append=True,
                compress=temp2_fh.name,
                lines=11,
                digest_records=True,
            )
            name = temp2_fh.name

        assert res3 == res.replace("comp_2.cdxj.gz", name)

    def test_warc_index_add_custom_fields(self):
        res = self.index_file("example.warc.gz", fields="method,referrer,http:date")

        exp = """\
com,example)/ 20170306040206 {"url": "http://example.com/", "mime": "text/html", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "1242", "offset": "784", "filename": "example.warc.gz", "method": "GET", "referrer": "https://webrecorder.io/temp-MJFXHZ4S/temp/recording-session/record/http://example.com/", "http:date": "Mon, 06 Mar 2017 04:02:06 GMT"}
com,example)/ 20170306040348 {"url": "http://example.com/", "mime": "warc/revisit", "status": "200", "digest": "sha1:G7HRM7BGOKSKMSXZAHMUQTTV53QOFSMK", "length": "585", "offset": "2635", "filename": "example.warc.gz", "http:date": "Mon, 06 Mar 2017 04:03:48 GMT"}
"""
        assert res == exp

    def test_warc_index_custom_fields_1(self):
        res = self.index_file(
            "example.warc.gz",
            records="response,request,revisit",
            replace_fields="warc-type",
        )

        exp = """\
com,example)/ 20170306040206 {"warc-type": "response"}
com,example)/ 20170306040206 {"warc-type": "request"}
com,example)/ 20170306040348 {"warc-type": "revisit"}
com,example)/ 20170306040348 {"warc-type": "request"}
"""
        assert res == exp

    def test_warc_index_custom_fields_2(self):
        res = self.index_file(
            "cc.warc.gz", records="all", replace_fields="method,mime,warc-type,date"
        )

        exp = """\
org,commoncrawl)/ 20170722005011 {"method": "GET", "warc-type": "request"}
org,commoncrawl)/ 20170722005011 {"method": "GET", "mime": "text/html", "warc-type": "response"}
org,commoncrawl)/ 20170722005011 {"mime": "application/warc-fields", "warc-type": "metadata"}
"""
        assert res == exp

    def test_cdxj_empty(self):
        output = StringIO()

        empty = BytesIO()

        opts = {"filename": "empty.warc.gz"}

        write_cdx_index(output, empty, opts)

        assert output.getvalue() == ""

    def test_cdxj_middle_empty_records(self):
        empty_gzip_record = b"\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00"

        new_warc = BytesIO()

        with open(os.path.join(TEST_DIR, "example.warc.gz"), "rb") as fh:
            new_warc.write(empty_gzip_record)
            new_warc.write(fh.read())
            new_warc.write(empty_gzip_record)
            new_warc.write(empty_gzip_record)
            fh.seek(0)
            new_warc.write(fh.read())

        new_warc.seek(0)

        output = StringIO()
        opts = {"filename": "empty.warc.gz"}

        write_cdx_index(output, new_warc, opts)

        lines = output.getvalue().rstrip().split("\n")

        assert len(lines) == 4, lines


class CustomIndexer(CDXJIndexer):
    def process_index_entry(self, it, record, *args):
        type_ = record.rec_headers.get("WARC-Type")
        if type_ == "response" and record.http_headers.get("Content-Type").startswith(
            "text/html"
        ):
            assert record.buffered_stream.read() != b""


def test_custom_indexer():
    output = StringIO()
    indexer = CustomIndexer(
        output=output,
        inputs=[os.path.join(TEST_DIR, "example.warc.gz")],
        fields="referrer",
    )

    assert indexer.collect_records

    indexer.process_all()



import hashlib
import shutil
import tempfile

from cdxj_indexer.postquery import append_method_query_from_req_resp


BUFF_SIZE = 1024 * 64


# ============================================================================
def buffering_record_iter(
    record_iter, post_append=False, digest_reader=None, url_key_func=None
):
    prev_record = None

    for record in record_iter:
        buffer_record_content(record)

        record.file_offset = record_iter.get_record_offset()
        record.file_length = record_iter.get_record_length()

        if digest_reader:
            curr = digest_reader.tell()
            digest_reader.seek(record.file_offset)
            record_digest, digest_length = digest_block(
                digest_reader, record.file_length
            )
            digest_reader.seek(curr)

            if digest_length != record.file_length:
                raise Exception(
                    "Digest block mismatch, expected {0}, got {1}",
                    record.file_length,
                    len(buff),
                )

            record.record_digest = record_digest

        req, resp = concur_req_resp(prev_record, record)

        if not req or not resp:
            if prev_record:
                yield prev_record
                prev_record.buffered_stream.close()
            prev_record = record
            continue

        join_req_resp(req, resp, post_append, url_key_func)

        yield prev_record
        prev_record.buffered_stream.close()
        yield record
        record.buffered_stream.close()
        prev_record = None

    if prev_record:
        yield prev_record
        prev_record.buffered_stream.close()


# ============================================================================
def concur_req_resp(rec_1, rec_2):
    if not rec_1 or not rec_2:
        return None, None

    if rec_1.rec_headers.get_header("WARC-Target-URI") != rec_2.rec_headers.get_header(
        "WARC-Target-URI"
    ):
        return None, None

    if rec_2.rec_headers.get_header(
        "WARC-Concurrent-To"
    ) != rec_1.rec_headers.get_header("WARC-Record-ID"):
        return None, None

    if rec_1.rec_type == "response" and rec_2.rec_type == "request":
        req = rec_2
        resp = rec_1

    elif rec_1.rec_type == "request" and rec_2.rec_type == "response":
        req = rec_1
        resp = rec_2

    else:
        return None, None

    return req, resp


# ============================================================================
def buffer_record_content(record):
    spool = tempfile.SpooledTemporaryFile()
    shutil.copyfileobj(record.content_stream(), spool)
    spool.seek(0)
    record.buffered_stream = spool


# ============================================================================
def join_req_resp(req, resp, post_append, url_key_func=None):
    resp.req = req

    method = req.http_headers.protocol
    if post_append and method.upper() in ("POST", "PUT"):
        url = req.rec_headers.get_header("WARC-Target-URI")
        query, append_str = append_method_query_from_req_resp(req, resp)
        resp.method = method.upper()
        resp.requestBody = query
        resp.urlkey = url + append_str
        if url_key_func:
            resp.urlkey = url_key_func(resp.urlkey)
        req.urlkey = resp.urlkey


# ============================================================================
def digest_block(reader, length):
    count = 0
    hasher = hashlib.sha256()

    while length > 0:
        buff = reader.read(min(BUFF_SIZE, length))
        if not buff:
            break
        hasher.update(buff)
        length -= len(buff)
        count += len(buff)

    return "sha256:" + hasher.hexdigest(), count



from __future__ import absolute_import
import json
import surt
import logging
import os
import re
import sys
import zlib
import hashlib

from argparse import ArgumentParser, RawTextHelpFormatter
from io import BytesIO
from copy import copy


from warcio.indexer import Indexer
from warcio.timeutils import iso_date_to_timestamp
from warcio.warcwriter import BufferWARCWriter
from warcio.archiveiterator import ArchiveIterator
from warcio.utils import open_or_default

from cdxj_indexer.bufferiter import buffering_record_iter


# ============================================================================
class CDXJIndexer(Indexer):
    field_names = {
        "warc-target-uri": "url",
        "http:status": "status",
        "warc-payload-digest": "digest",
        "req.http:referer": "referrer",
        "req.http:method": "method",
        "record-digest": "recordDigest",
    }

    inv_field_names = {k: v for v, k in field_names.items()}

    DEFAULT_FIELDS = [
        "warc-target-uri",
        "mime",
        "http:status",
        "warc-payload-digest",
        "length",
        "offset",
        "filename",
    ]

    DEFAULT_RECORDS = ["response", "revisit", "resource", "metadata"]

    ALLOWED_EXT = (".arc", ".arc.gz", ".warc", ".warc.gz")

    RE_SPACE = re.compile(r"[;\s]")

    DEFAULT_NUM_LINES = 300

    def __init__(
        self,
        output,
        inputs,
        post_append=False,
        sort=False,
        compress=None,
        lines=DEFAULT_NUM_LINES,
        data_out_name=None,
        filename=None,
        fields=None,
        replace_fields=None,
        records=None,
        verify_http=False,
        dir_root=None,
        digest_records=False,
        **kwargs
    ):

        if isinstance(inputs, str) or hasattr(inputs, "read"):
            inputs = [inputs]

        inputs = iter_file_or_dir(inputs)

        self.digest_records = digest_records
        fields = self._parse_fields(fields, replace_fields)

        super(CDXJIndexer, self).__init__(
            fields, inputs, output, verify_http=verify_http
        )
        self.writer = None

        self.curr_filename = None
        self.force_filename = filename
        self.post_append = post_append
        self.dir_root = dir_root

        self.num_lines = lines
        self.sort = sort
        self.compress = compress
        self.data_out_name = data_out_name

        self.include_records = records
        if self.include_records == "all":
            self.include_records = None
        elif self.include_records:
            self.include_records = self.include_records.split(",")
        else:
            self.include_records = self.DEFAULT_RECORDS

        self.collect_records = self.post_append or any(
            field.startswith("req.http:") for field in self.fields
        )
        self.record_parse = True

    def _parse_fields(self, fields=None, replace_fields=None):
        add_fields = replace_fields
        if add_fields:
            fields = []
        else:
            add_fields = fields
            fields = copy(self.DEFAULT_FIELDS)

        if self.digest_records and "record-digest" not in fields:
            fields.append("record-digest")

        if add_fields:
            add_fields = add_fields.split(",")
            for field in add_fields:
                fields.append(self.inv_field_names.get(field, field))

        return fields

    def get_field(self, record, name, it, filename):
        if name == "mime":
            if record.rec_type == "revisit":
                return "warc/revisit"
            elif record.rec_type in ("response", "request"):
                name = "http:content-type"
            else:
                name = "content-type"

            value = super(CDXJIndexer, self).get_field(record, name, it, filename)
            if value:
                value = self.RE_SPACE.split(value, 1)[0].strip()

            return value

        if name == "filename":
            return self.curr_filename

        if self.collect_records:
            if name == "offset":
                return str(record.file_offset)
            elif name == "length":
                return str(record.file_length)
            elif name == "record-digest":
                return str(record.record_digest)
            elif name.startswith("req.http:"):
                value = self._get_req_field(name, record)
                if value:
                    return value

        value = super(CDXJIndexer, self).get_field(record, name, it, filename)

        if name == "warc-payload-digest":
            value = self._get_digest(record, name)

        return value

    def _get_req_field(self, name, record):
        if hasattr(record, "req"):
            req = record.req
        elif record.rec_type == "request":
            req = record
        else:
            return None

        if name == "req.http:method":
            return req.http_headers.protocol
        else:
            return req.http_headers.get_header(name[9:])

    def process_all(self):
        data_out = None

        with open_or_default(self.output, "wt", sys.stdout) as fh:
            if self.compress:
                if isinstance(self.compress, str):
                    data_out = open(self.compress, "wb")
                    if os.path.splitext(self.compress)[1] == "":
                        self.compress += ".cdxj.gz"

                    fh = CompressedWriter(
                        fh,
                        data_out=data_out,
                        data_out_name=self.compress,
                        num_lines=self.num_lines,
                        digest_records=self.digest_records,
                    )
                else:
                    fh = CompressedWriter(
                        fh,
                        data_out=self.compress,
                        data_out_name=self.data_out_name,
                        num_lines=self.num_lines,
                        digest_records=self.digest_records,
                    )

            if self.sort:
                fh = SortingWriter(fh)

            self.output = fh

            super().process_all()

            if self.sort or self.compress:
                fh.flush()
                if data_out:
                    data_out.close()

    def _resolve_rel_path(self, filename):
        if not self.dir_root:
            return os.path.basename(filename)

        path = os.path.relpath(filename, self.dir_root)
        if os.path.sep != "/":  # pragma: no cover
            path = path.replace(os.path.sep, "/")
        return path

    def process_one(self, input_, output, filename):
        self.curr_filename = self.force_filename or self._resolve_rel_path(filename)

        it = self._create_record_iter(input_)

        self._write_header(output, filename)

        if self.collect_records:
            digest_reader = input_ if self.digest_records else None
            wrap_it = buffering_record_iter(
                it,
                post_append=self.post_append,
                digest_reader=digest_reader,
                url_key_func=self.get_url_key,
            )
        else:
            wrap_it = it

        for record in wrap_it:
            if not self.include_records or self.filter_record(record):
                self.process_index_entry(it, record, filename, output)

    def filter_record(self, record):
        if not record.rec_type in self.include_records:
            return False

        if (
            self.include_records == self.DEFAULT_RECORDS
            and record.rec_type in ("resource", "metadata")
            and record.rec_headers.get_header("Content-Type")
            == "application/warc-fields"
        ):
            return False

        return True

    def _get_digest(self, record, name):
        value = record.rec_headers.get(name)
        if not value:
            if not self.writer:
                self.writer = BufferWARCWriter()

            self.writer.ensure_digest(record, block=False, payload=True)
            value = record.rec_headers.get(name)

        return value

    def _write_line(self, out, index, record, filename):
        url = index.get("url")
        if not url:
            url = record.rec_headers.get("WARC-Target-URI")

        dt = record.rec_headers.get("WARC-Date")

        ts = iso_date_to_timestamp(dt)

        if hasattr(record, "urlkey"):
            urlkey = record.urlkey
        else:
            urlkey = self.get_url_key(url)

        if hasattr(record, "requestBody"):
            index["requestBody"] = record.requestBody
        if hasattr(record, "method"):
            index["method"] = record.method

        self._do_write(urlkey, ts, index, out)

    def _do_write(self, urlkey, ts, index, out):
        out.write(urlkey + " " + ts + " " + json.dumps(index) + "\n")

    def get_url_key(self, url):
        try:
            return surt.surt(url)
        except:  # pragma: no coverage
            return url


# ============================================================================
class CDXLegacyIndexer(CDXJIndexer):
    def _do_write(self, urlkey, ts, index, out):
        index["urlkey"] = urlkey
        index["timestamp"] = ts

        line = " ".join(index.get(field, "-") for field in self.CDX_FIELDS)
        out.write(line + "\n")

    def _write_header(self, out, filename):
        out.write(self.CDX_HEADER + "\n")

    def get_field(self, record, name, it, filename):
        value = super().get_field(record, name, it, filename)

        if name == "warc-payload-digest":
            value = value.split(":")[-1]

        return value


# ============================================================================
class CDX11Indexer(CDXLegacyIndexer):
    CDX_HEADER = " CDX N b a m s k r M S V g"

    CDX_FIELDS = [
        "urlkey",
        "timestamp",
        "url",
        "mime",
        "status",
        "digest",
        "redirect",
        "meta",
        "length",
        "offset",
        "filename",
    ]


# ============================================================================
class CDX09Indexer(CDXLegacyIndexer):
    CDX_HEADER = " CDX N b a m s k r V g"

    CDX_FIELDS = [
        "urlkey",
        "timestamp",
        "url",
        "mime",
        "status",
        "digest",
        "redirect",
        "offset",
        "filename",
    ]


# ============================================================================
class SortingWriter:
    def __init__(self, out):
        self.out = out
        self.sortedlist = []

    def write(self, line):
        self.sortedlist.append(line)

    def flush(self):
        self.sortedlist.sort()
        lastline = None
        for line in self.sortedlist:
            if lastline != line:
                self.out.write(line)
            lastline = line

        self.out.flush()


# ============================================================================
class CompressedWriter:
    def __init__(
        self,
        index_out,
        data_out,
        num_lines=CDXJIndexer.DEFAULT_NUM_LINES,
        data_out_name="",
        digest_records=False,
    ):
        self.index_out = index_out
        self.data_out = data_out
        self.data_out_name = data_out_name
        self.digest_records = digest_records

        self.block = []
        self.offset = 0
        self.prefix = ""
        self.num_lines = num_lines

    def write_header(self):
        meta = json.dumps({"format": "cdxj-gzip-1.0", "filename": self.data_out_name})

        self.index_out.write("!meta 0 {0}\n".format(meta))

    def write(self, line):
        if not len(self.block):
            self.prefix = line.split("{", 1)[0].strip()
            if not self.offset:
                self.write_header()

        self.block.append(line)

        if len(self.block) == self.num_lines:
            self.flush()

    def get_index_json(self, length, digest):
        data = {"offset": self.offset, "length": length}
        if digest:
            data["digest"] = digest

        return json.dumps(data) + "\n"

    def flush(self):
        comp = zlib.compressobj(wbits=16 + zlib.MAX_WBITS)
        compressed = comp.compress("".join(self.block).encode("utf-8"))
        compressed += comp.flush()

        length = len(compressed)
        digest = (
            "sha256:" + hashlib.sha256(compressed).hexdigest()
            if self.digest_records
            else None
        )
        line = self.prefix + " " + self.get_index_json(length, digest)
        self.index_out.write(line)
        self.data_out.write(compressed)
        self.offset += length
        self.block = []


# ============================================================================
def main(args=None):
    parser = ArgumentParser(
        description="cdx_indexer", formatter_class=RawTextHelpFormatter
    )

    parser.add_argument("inputs", nargs="+")
    parser.add_argument("-o", "--output")

    group = parser.add_mutually_exclusive_group()
    group.add_argument("-9", "--cdx09", action="store_true")

    group.add_argument("-11", "--cdx11", action="store_true")

    group.add_argument("-f", "--fields")
    group.add_argument("-rf", "--replace-fields")

    parser.add_argument("--records")

    parser.add_argument("--dir-root")

    parser.add_argument("-p", "--post-append", action="store_true")

    parser.add_argument("-s", "--sort", action="store_true")

    parser.add_argument("-c", "--compress")

    parser.add_argument(
        "-l", "--lines", type=int, default=CDXJIndexer.DEFAULT_NUM_LINES
    )

    parser.add_argument("-d", "--digest-records", action="store_true")

    cmd = parser.parse_args(args=args)

    write_cdx_index(cmd.output, cmd.inputs, vars(cmd))


def write_cdx_index(output, inputs, opts):
    if opts.get("cdx11"):
        cls = CDX11Indexer
    elif opts.get("cdx09"):
        cls = CDX09Indexer
    else:
        cls = CDXJIndexer

    opts.pop("output", "")
    opts.pop("inputs", "")

    indexer = cls(output, inputs, **opts)
    indexer.process_all()
    return indexer


# =================================================================
def iter_file_or_dir(inputs, recursive=True):
    for input_ in inputs:
        if not isinstance(input_, str) or not os.path.isdir(input_):
            yield input_
            continue

        for root, dirs, files in os.walk(input_):
            for filename in files:
                if filename.endswith(CDXJIndexer.ALLOWED_EXT):
                    full_path = os.path.join(root, filename)
                    yield full_path


# ============================================================================
if __name__ == "__main__":  # pragma: no cover
    main()



from warcio.utils import to_native_str

from urllib.parse import unquote_plus, urlencode
from io import BytesIO

from cdxj_indexer.amf import amf_parse

import base64
import cgi
import json
import sys

MAX_QUERY_LENGTH = 4096

# ============================================================================
def append_method_query_from_req_resp(req, resp):
    len_ = req.http_headers.get_header("Content-Length")
    content_type = req.http_headers.get_header("Content-Type")
    stream = req.buffered_stream
    stream.seek(0)

    url = req.rec_headers.get_header("WARC-Target-URI")
    method = req.http_headers.protocol
    return append_method_query(method, content_type, len_, stream, url)


# ============================================================================
def append_method_query(method, content_type, len_, stream, url):
    # if method == 'GET':
    #    return '', ''

    if method == "POST" or method == "PUT":
        query = query_extract(content_type, len_, stream, url)
    else:
        query = ""

    if "?" not in url:
        append_str = "?"
    else:
        append_str = "&"

    append_str += "__wb_method=" + method
    if query:
        append_str += "&" + query

    return query, append_str


# ============================================================================
def query_extract(mime, length, stream, url):
    """
    Extract a url-encoded form POST/PUT from stream
    content length, return None
    Attempt to decode application/x-www-form-urlencoded or multipart/*,
    otherwise read whole block and b64encode
    """
    query_data = b""

    try:
        length = int(length)
    except (ValueError, TypeError):
        if length is None:
            length = 8192
        else:
            return

    while length > 0:
        buff = stream.read(length)

        length -= len(buff)

        if not buff:
            break

        query_data += buff

    if not mime:
        mime = ""

    query = ""

    def handle_binary(query_data):
        query = base64.b64encode(query_data)
        query = to_native_str(query)
        query = "__wb_post_data=" + query
        return query

    if mime.startswith("application/x-www-form-urlencoded"):
        try:
            query = to_native_str(query_data.decode("utf-8"))
            query = unquote_plus(query)
        except UnicodeDecodeError:
            query = handle_binary(query_data)

    elif mime.startswith("multipart/"):
        env = {
            "REQUEST_METHOD": "POST",
            "CONTENT_TYPE": mime,
            "CONTENT_LENGTH": len(query_data),
        }

        args = dict(fp=BytesIO(query_data), environ=env, keep_blank_values=True)

        args["encoding"] = "utf-8"

        try:
            data = cgi.FieldStorage(**args)
        except ValueError:
            # Content-Type multipart/form-data may lack "boundary" info
            query = handle_binary(query_data)
        else:
            values = []
            for item in data.list:
                values.append((item.name, item.value))

            query = urlencode(values, True)

    elif mime.startswith("application/json"):
        try:
            query = json_parse(query_data)
        except Exception as e:
            if query_data:
                try:
                    sys.stderr.write(
                        "Error parsing: " + query_data.decode("utf-8") + "\n"
                    )
                except:
                    pass

            query = ""

    elif mime.startswith("text/plain"):
        try:
            query = json_parse(query_data)
        except Exception as e:
            query = handle_binary(query_data)

    elif mime.startswith("application/x-amf"):
        query = amf_parse(query_data)
    else:
        query = handle_binary(query_data)

    if query:
        query = query[:MAX_QUERY_LENGTH]

    return query


def json_parse(string):
    data = {}
    dupes = {}

    def get_key(n):
        if n not in data:
            return n

        if n not in dupes:
            dupes[n] = 1

        dupes[n] += 1
        return n + "." + str(dupes[n]) + "_"

    def _parser(json_obj, name=""):
        if isinstance(json_obj, dict):
            for n, v in json_obj.items():
                _parser(v, n)

        elif isinstance(json_obj, list):
            for v in json_obj:
                _parser(v, name)

        elif name:
            data[get_key(name)] = str(json_obj)

    try:
        _parser(json.loads(string))
    except json.decoder.JSONDecodeError:
        if b"\n" in string:
            for line in string.split(b"\n"):
                _parser(json.loads(line))
        else:
            raise

    return urlencode(data)



#!/usr/bin/env python
# vim: set sw=4 et:

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import glob

__version__ = "1.4.3"


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        # should work with setuptools <18, 18 18.5
        self.test_suite = " "

    def run_tests(self):
        import pytest
        import sys
        import os

        errcode = pytest.main(
            [
                "--doctest-modules",
                "./cdxj_indexer",
                "--cov",
                "cdxj_indexer",
                "-v",
                "test/",
            ]
        )
        sys.exit(errcode)


setup(
    name="cdxj_indexer",
    version=__version__,
    author="Ilya Kreymer",
    author_email="ikreymer@gmail.com",
    license="Apache 2.0",
    packages=find_packages(),
    url="https://github.com/webrecorder/cdxj-indexer",
    description="CDXJ Indexer for WARC and ARC files",
    long_description=open("README.rst").read(),
    provides=[
        "cdxj_indexer",
    ],
    install_requires=[
        "warcio",
        "surt",
        # temp fix for requests
        "idna<3.0",
        "py3amf",
    ],
    zip_safe=True,
    entry_points="""
        [console_scripts]
        cdxj-indexer=cdxj_indexer.main:main
    """,
    cmdclass={"test": PyTest},
    test_suite="",
    tests_require=[
        "pytest",
        "pytest-cov",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
)
